/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.hayabusa.taglib;

import static org.opengion.fukurou.util.StringUtil.nval;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;

import javax.servlet.http.HttpServletRequest;

import org.opengion.fukurou.util.StringUtil;
import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.hayabusa.common.HybsSystemException;
import org.opengion.hayabusa.db.DBColumn;
import org.opengion.hayabusa.db.DBTableModel;
import org.opengion.hayabusa.db.DBTableModelUtil;
import org.opengion.hayabusa.servlet.MultipartRequest;

/**
 * クライアントのファイルをサーバーにアップロードするタグです。
 *
 * アップロード後の属性は、DBTableModel に格納することにより、通常のデータと
 * 同様の取り出し方が可能です。
 * また、通常のファイルアップロード時の、form で使用する、enctype="multipart/form-data"
 * を指定した場合の、他のリクエスト情報も、{&#064;XXXX} 変数で取り出すことが可能です。
 *
 * この upload タグでは、アップロード後に、指定のファイル名に変更する機能があります。
 * file 登録ダイアログで指定した name に、"_NEW" という名称を付けたリクエスト値を
 * ファイルのアップロードと同時に送信することで、この名前にファイルを付け替えます。
 * また、アップロード後のファイル名は、name 指定の名称で、取り出せます。
 * クライアントから登録したオリジナルのファイル名は、name に、"_ORG" という名称
 * で取り出すことが可能です。
 *
 * 通常では、これらのパラメータを、RequestAttribute 変数にセットしますので、{&#064;XXXX}で
 * 取り出すことが可能になります。さらに、"KEY","VALUE","ISFILE" のカラムを持った、
 * DBTableModel にセットします。
 *
 * 新機能として、columns を指定する事で、columns のカラムを持つ DBTableModel にセットします。
 * その場合は、カラム名_01 ～ カラム名_99 のように、アンダーバーで列データとなるキーを定義してください。
 * アンダーバーがない場合は、カラムだけ作成されます。カラム名と同じリクエストがあれば、
 * すべてのレコードに同じ値がセットされます。
 *
 * 新ファイル名に拡張子が設定されていない場合は、オリジナルファイル名の拡張子をセットします。
 *
 * @og.formSample
 * ●形式：&lt;og:upload fileURL="…" maxPostSize="…" /&gt;
 * ●body：なし
 *
 * ●Tag定義：
 *   &lt;og:upload
 *       fileURL            【TAG】ファイルをアップロードするディレクトリを指定します (初期値:FILE_URL[=filetemp/])
 *       filename           【TAG】(通常使いません)ファイルを作成するときのファイル名をセットします
 *       maxPostSize        【TAG】最大転送サイズ(Byte)を指定します(初期値:10485760) 0,またはマイナスで無制限です。
 *       scope              【TAG】キャッシュする場合のスコープ[request/page/session/applicaton]を指定します(初期値:session)
 *       tableId            【TAG】(通常使いません)sessionから所得する DBTableModelオブジェクトの ID
 *       columns            【TAG】DBTableModel作成時に、指定のカラムの"_01"～"_99"の添え字をレコードとして作成します。
 *       useBackup          【TAG】ファイルアップロード時に、すでに同名のファイルが存在した場合に、バックアップ処理(renameTo)するかどうか[true/false]を指定します(初期値:false)
 *       language           【TAG】タグ内部で使用する言語コード[ja/en/zh/…]を指定します
 *       debug              【TAG】デバッグ情報を出力するかどうか[true/false]を指定します(初期値:false)
 *   /&gt;
 *
 * ●使用例 :
 * 【query.jsp】
 *       &lt;form method="POST" action="result.jsp" enctype="multipart/form-data" target="RESULT"&gt;
 *       &lt;table summary="layout" &gt;
 *       &lt;tr&gt;&lt;og:input type="text" name="submitter" value="{&#064;USER.JNAME}" size="20" msg="MSG0014" /&gt;&lt;/tr&gt;
 *       &lt;tr&gt;
 *           &lt;og:input type="file" name="file_01"      size="30" msg="MSG0015" /&gt;
 *           &lt;og:input             name="file_01_NEW"  size="10" lbl="FILENAME" /&gt;
 *       &lt;/tr&gt;&lt;tr&gt;
 *           &lt;og:input type="file" name="file_02"      size="30" msg="MSG0015" /&gt;
 *           &lt;og:input             name="file_02_NEW"  size="10" lbl="FILENAME" /&gt;
 *       &lt;/tr&gt;&lt;tr&gt;
 *           &lt;og:input type="file" name="file_03"      size="30" msg="MSG0015" /&gt;
 *           &lt;og:input             name="file_03_NEW"  size="10" lbl="FILENAME" /&gt;
 *       &lt;/tr&gt;&lt;tr&gt;
 *           &lt;og:column name="writable"     value="false"  /&gt;
 *       &lt;/tr&gt;
 *       &lt;/table&gt;
 *
 * 【result.jsp】
 *       &lt;og:upload
 *           fileURL     = "{&#064;USER.ID}"
 *       /&gt;
 *       &lt;br /&gt;
 *       &lt;og:message msg="MSG0003" comment="ファイルの登録が完了しました。" /&gt;
 *
 *       &lt;og:view
 *           command      = "NEW"
 *           viewFormType = "HTMLTable"
 *           writable     = "{&#064;writable}"
 *       /&gt;
 *
 *       &lt;table&gt;
 *       &lt;tr&gt;&lt;og:input name="submitter"   value="{&#064;submitter}"   /&gt;&lt;/tr&gt;
 *       &lt;tr&gt;&lt;og:input name="writable"    value="{&#064;writable}"    /&gt;&lt;/tr&gt;
 *       &lt;tr&gt;&lt;og:input name="directory"   value="{&#064;directory}"   /&gt;&lt;/tr&gt;
 *       &lt;tr&gt;&lt;og:input name="file_01"     value="{&#064;file_01}"     /&gt;&lt;/tr&gt;
 *       &lt;tr&gt;&lt;og:input name="file_01_NEW" value="{&#064;file_01_NEW}" /&gt;&lt;/tr&gt;
 *       &lt;tr&gt;&lt;og:input name="file_01_ORG" value="{&#064;file_01_ORG}" /&gt;&lt;/tr&gt;
 *       &lt;tr&gt;&lt;og:input name="file_02"     value="{&#064;file_02}"     /&gt;&lt;/tr&gt;
 *       &lt;tr&gt;&lt;og:input name="file_02_NEW" value="{&#064;file_02_NEW}" /&gt;&lt;/tr&gt;
 *       &lt;tr&gt;&lt;og:input name="file_02_ORG" value="{&#064;file_02_ORG}" /&gt;&lt;/tr&gt;
 *       &lt;tr&gt;&lt;og:input name="file_03"     value="{&#064;file_03}"     /&gt;&lt;/tr&gt;
 *       &lt;tr&gt;&lt;og:input name="file_03_NEW" value="{&#064;file_03_NEW}" /&gt;&lt;/tr&gt;
 *       &lt;tr&gt;&lt;og:input name="file_03_ORG" value="{&#064;file_03_ORG}" /&gt;&lt;/tr&gt;
 *       &lt;/table&gt;
 *
 * 【result.jsp】
 *       &lt;og:upload
 *           fileURL     = "{&#064;USER.ID}"
 *           columns     = "submitter,file"
 *       /&gt;
 *       &lt;br /&gt;
 *       &lt;og:message msg="MSG0003" comment="ファイルの登録が完了しました。" /&gt;
 *
 *       &lt;og:view
 *           command      = "NEW"
 *           viewFormType = "HTMLTable"
 *           writable     = "{&#064;writable}"
 *       /&gt;
 *
 * @og.group ファイル入力
 *
 * @version  4.0
 * @author	 Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public class FileUploadTag extends CommonTagSupport {
	//* このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "5.6.6.1 (2013/07/12)" ;

	private static final long serialVersionUID = 566120130712L ;

	// 3.5.2.0 (2003/10/20) カラム名に、ISFILEを追加。
	private static final String[] names = new String[] { "KEY","VALUE","ISFILE" };
	private static final String   ENCODE = "UTF-8";			// 3.5.2.0 (2003/10/20) 廃止
	private String	fileURL 	= HybsSystem.sys( "FILE_URL" );
	private String  filename    = null;						// 3.5.4.2 (2003/12/15)

	private int 	maxPostSize = 10*1024*1024;				// 最大ファイル容量  10MB
	private String	tableId 	= HybsSystem.TBL_MDL_KEY ;

	// 5.6.5.2 (2013/06/21) DBTableModel作成時に、指定のカラムの"_01"～"_99"の添え字をレコードとして作成します。
	private String		columns		= null;
	// 5.6.5.3 (2013/06/28) ファイルアップロード時に、すでに同名のファイルが存在した場合に、バックアップ処理(renameTo)するかどうか[true/false]を指定します(初期値:false)
	private boolean	useBackup	= false;

	/**
	 * Taglibの終了タグが見つかったときに処理する doEndTag() を オーバーライドします。
	 *
	 * @og.rev 2.2.0.0 (2002/12/17) 中国語(国際化)対応 エンコードの取得方法変更
	 * @og.rev 3.1.1.2 (2003/04/04) Tomcat4.1 対応。release2() を doEndTag()で呼ぶ。
	 * @og.rev 3.1.3.0 (2003/04/10) UTF-8 決め打ちのエンコード情報を取得する。
	 * @og.rev 3.5.2.0 (2003/10/20) scope 属性を追加。
	 * @og.rev 3.5.4.2 (2003/12/15) ファイル名を指定できるようにします。
	 * @og.rev 3.6.0.8 (2004/11/19) DBTableModel をセーブする時に、トランザクションチェックを行います。
	 * @og.rev 3.7.1.1 (2005/05/23) フォルダがない場合は、複数階層分のフォルダを自動で作成します。
	 * @og.rev 3.8.1.3A (2006/01/30) 新ファイル名にオリジナルファイル名の拡張子をセットします
	 * @og.rev 5.3.7.0 (2011/07/01) エラーメッセージ内容変更
	 * @og.rev 5.6.5.2 (2013/06/21) columns 属性の追加
	 * @og.rev 5.6.5.3 (2013/06/28) useBackup 属性の追加
	 *
	 * @return	後続処理の指示
	 */
	@Override
	public int doEndTag() {
		debugPrint();		// 4.0.0 (2005/02/28)
		startQueryTransaction( tableId );		// 3.6.0.8 (2004/11/19)
		HttpServletRequest request = (HttpServletRequest)getRequest();

		try {
			String directory = HybsSystem.url2dir( fileURL );
			File dir = new File(directory);
			if( ! dir.exists() && ! dir.mkdirs() ) {
				String errMsg = "ディレクトリの作成に失敗しました。[" + directory + "]";
				throw new HybsSystemException( errMsg );
			}

			// 3.8.1.3A (2006/01/30) 新ファイル名にオリジナルファイル名の拡張子をセットします
//			MultipartRequest multi = new MultipartRequest( request,directory,maxPostSize,ENCODE,filename );
			// 5.6.5.3 (2013/06/28) useBackup 属性の追加
			MultipartRequest multi = new MultipartRequest( request,directory,maxPostSize,ENCODE,filename,useBackup );
			// 5.6.5.2 (2013/06/21) columns 属性の追加
//			DBTableModel table = makeDBTable( multi );
			DBTableModel table = null;
			if( columns == null ) {						// 5.6.5.2 (2013/06/21) columns 属性の追加
				table = makeDBTable( multi );
			}
			else {
				table = makeDBTableFromClms( multi );
			}

			// 3.5.2.0 (2003/10/20) scope 属性を追加。
			// 3.6.0.8 (2004/11/19) トランザクションチェックを行います。
			if( ! commitTableObject( tableId, table ) ) {
				jspPrint( "FileUploadTag Query処理が割り込まれました。DBTableModel は登録しません。" );
				return (SKIP_PAGE);
			}

		} catch(IOException ex) {
//			String errMsg = "ファイルの取り扱い中にエラーが発生しました。"
//					+ toString() + HybsSystem.CR
//					+ "FileURL=" + fileURL + HybsSystem.CR
//					+ ex.getMessage();				// 5.1.8.0 (2010/07/01) errMsg 修正
			String errMsg = "ファイル登録エラー！"
				+ ex.getMessage() + HybsSystem.CR	// 5.3.7.0 (2011/07/01) errMsg 修正
				+ "(" + toString() + HybsSystem.CR
				+ "FileURL=" + fileURL + ")";
			throw new HybsSystemException( errMsg,ex );		// 3.5.5.4 (2004/04/15) 引数の並び順変更
		}

		return(EVAL_PAGE);
	}

	/**
	 * タグリブオブジェクトをリリースします。
	 * キャッシュされて再利用されるので、フィールドの初期設定を行います。
	 *
	 * @og.rev 2.0.0.4 (2002/09/27) カスタムタグの release() メソッドを、追加
	 * @og.rev 3.0.1.1 (2003/03/06) columns を廃止
	 * @og.rev 3.1.1.2 (2003/04/04) Tomcat4.1 対応。release2() を doEndTag()で呼ぶ。
	 * @og.rev 3.5.4.2 (2003/12/15) ファイル名を指定できるようにします。
	 * @og.rev 5.6.5.2 (2013/06/21) columns 属性の追加
	 * @og.rev 5.6.5.2 (2013/06/21) useBackup 属性の追加
	 *
	 */
	@Override
	protected void release2() {
		super.release2();
		fileURL 		= HybsSystem.sys( "FILE_URL" );
		maxPostSize		= 10*1024*1024;				// 最大ファイル容量  10MB
		tableId 		= HybsSystem.TBL_MDL_KEY ;
		filename		= null;						// 3.5.4.2 (2003/12/15)
		columns			= null;						// 5.6.5.2 (2013/06/21)
		useBackup		= false;					// 5.6.5.3 (2013/06/28) 
	}

	/**
	 * ファイルアップロードの実行結果を DBTableModel に記述します。
	 *
	 * ここでは、"KEY","VALUE","ISFILE" のカラムに対して、値を設定していきます。
	 * 同時に、RequestAttribute 変数に、これらの値をセットすることで、
	 * {&#064;XXXX} で値が取り出せる様にしています。
	 *
	 * @og.rev 2.2.0.0 (2002/12/17) 中国語(国際化)対応 エンコードの取得方法変更
	 * @og.rev 3.0.1.1 (2003/03/06) request 情報から{&#064;XXXX} で値が取り出せる様に修正。
	 * @og.rev 3.5.2.0 (2003/10/20) カラム名(KEY,VALUE)に ISFILE を追加
	 * @og.rev 3.5.6.5 (2004/08/09) MultipartRequest 変更に伴なう修正(Enum変更、元ファイル名取得)
	 * @og.rev 3.5.6.6 (2004/08/23) 上記変更時のバグ修正。
	 * @og.rev 3.5.6.6 (2004/08/23) 元ファイルのキーを、XXXX_ORG にします。
	 * @og.rev 4.0.0.0 (2007/10/12) テーブルモデルの登録方法を変更
	 * @og.rev 5.3.2.0 (2011/02/01) チェック行のパラメーターはint配列側に変換して復元する。パラメーター名を保存する。
	 * @og.rev 5.4.4.2 (2012/02/03) CommonTagSupportと同様のチェックボックス判定を行う
	 *
	 * @param	 multi	   MultipartRequestオブジェクト
	 *
	 * @return	 テーブルモデル
	 */
	private DBTableModel makeDBTable( final MultipartRequest multi ) {

		DBTableModel table = DBTableModelUtil.newDBTable();

		table.init( names.length );

		for( int i=0; i<names.length; i++ ) {
			DBColumn dbColumn = getDBColumn( names[i] );
			table.setDBColumn( i,dbColumn );
		}

		String[] values ;		// 4.0.0.0 (2007/10/12)

		String[] files = multi.getFilenames();		// 3.6.0.0 (2004/09/17)
		for( int i=0; i<files.length; i++ ) {
			String name = files[i];
			File fin = multi.getFile(name);
			if( fin != null ) {
				String val = multi.getFilesystemName(name);

				// "KEY","VALUE","ISFILE" の順にデータを作成します。
				values = new String[] { name, val, "1" };
				table.addColumnValues( values );
				setRequestAttribute( name,val );

				String orgName = name + "_ORG" ;
				val  = multi.getOriginalFileName(name);	// 注意：取得は、送信名

				// "KEY","VALUE","ISFILE" の順にデータを作成します。
				values = new String[] { orgName, val, "2" };
				table.addColumnValues( values );
				setRequestAttribute( orgName,val );
			}
		}

		// "KEY","VALUE","ISFILE" の順にデータを作成します。
		values = new String[] { "directory", fileURL, "0" };
		table.addColumnValues( values );
		setRequestAttribute( "directory",fileURL );

		String[] params = multi.getParameterNames();
		for( int i=0; i<params.length; i++ ) {
			String name = params[i];
			// 5.3.2.0 (2011/02/01) チェック行のパラメーターはint配列側に変換
			if ( HybsSystem.ROW_SEL_KEY.equals( name ) ) {
				setRequestAttribute( name,multi.getIntParameters(name) );
			}
			else {
				// 5.6.5.2 (2013/06/21) チェックボックス配列の値取得を考慮した、MultipartRequest のパラメータ値取得
				String val = getParamVal( name,multi );
//				String val = multi.getParameter(name);
//				// "KEY","VALUE","ISFILE" の順にデータを作成します。
//				if( "0".equals(val) ){ // 5.4.4.2 チェックボックス配列対応。
//					String[] vals = multi.getParameters(name);
//					if( vals != null && vals.length > 1 ) {
//						for( int j=0; j<vals.length; j++ ) {
//							if( "1".equals( vals[j] ) ) {
//								val = "1";
//								break;
//							}
//						}
//					}
//				}

				values = new String[] { name, val, "0" };
				table.addColumnValues( values );
				setRequestAttribute( name,val );
			}
		}

		// 5.3.2.0 (2011/02/01) パラメーター名を保存する。
		String[] names = new String[ files.length + params.length ];
	    System.arraycopy( files, 0, names, 0, files.length );
	    System.arraycopy( params, 0, names, files.length, params.length );
	    setParameterNames( names );

		return table ;
	}

	/**
	 * ファイルアップロードの実行結果を 横持の DBTableModel に記述します。
	 * 
	 * この処理は、columns 属性を設定した場合のみとします。
	 * 
	 * DBTableModel作成時に、指定のカラムの"_01"～"_99"の添え字をレコードとして作成します。
	 * 現状は、"KEY","VALUE","ISFILE" のカラムに、データを縦持ちで作成しています。
 	 * これを、横持で作成しますが、カラムの末尾に、"_01"～"_99" までの添え字を
	 * 持つ場合は、これをレコードと認識させます。
	 * <del>添え字以外は、固定値になるので、DBTableModel に含めることはできません。</del>
	 * 添え字がない場合は、カラムだけ作成されます。カラム名と同じリクエストがあれば、
	 * すべてのレコードに同じ値がセットされます。
	 *
	 * @og.rev 5.6.5.2 (2013/06/21) 新規作成
	 * @og.rev 5.6.6.1 (2013/07/12) 添え字がない場合の処理の見直し
	 *
	 * @param	 multi	   MultipartRequestオブジェクト
	 *
	 * @return	 テーブルモデル
	 */
	private DBTableModel makeDBTableFromClms( final MultipartRequest multi ) {

		DBTableModel table = DBTableModelUtil.newDBTable();

		String[] clmNames = columns.split( "," );

		table.init( clmNames.length );

		// 値配列（１行分）
		String[] rowVal = new String[clmNames.length];

		for( int i=0; i<clmNames.length; i++ ) {
			String clm = clmNames[i] ;
			DBColumn dbColumn = getDBColumn( clm );
			table.setDBColumn( i,dbColumn );

			// 先に、カラム名と一致するパラメータを初期値としてセットしておきます。
			String val = getParamVal( clm,multi );

			// 5.6.6.1 (2013/07/12) ファイル名も同様に、あれば初期値セットしておきます。
			if( val == null ) {
				val = multi.getFilesystemName( clm );
			}
			if( val == null ) {
				val = multi.getOriginalFileName( clm );	// 注意：取得は、送信名
			}
			rowVal[i] = ( val == null ) ? "" : val ;
		}

		String[] files = multi.getFilenames();		// 3.6.0.0 (2004/09/17)
		for( int i=0; i<files.length; i++ ) {
			String name = files[i];
			File fin = multi.getFile(name);
			if( fin != null ) {
				// 5.6.6.1 (2013/07/12) 添え字がない場合の処理の見直し。先にレコードを作成
				String[] values = new String[clmNames.length];
				System.arraycopy( rowVal,0,values,0,values.length );		// 行にセットするに当たり、rowVal を values にコピーしておく。

				// ファイル名を Attribute で使えるようにセットしておく。
				String fval = multi.getFilesystemName(name);
				setRequestAttribute( name,fval );

				String orgName = name + "_ORG" ;
				String oval  = multi.getOriginalFileName(name);	// 注意：取得は、送信名
				setRequestAttribute( orgName,oval );

				// ファイルのキーを元に、添え字を検索します。
				int adrs = name.lastIndexOf( '_' );		// 添え字は、'_' で区切られます。
				// 5.6.6.1 (2013/07/12) 添え字がない場合の処理の見直し。後続処理を行う。
//				if( adrs < 0 ) { continue; }			// 添え字がない
				if( adrs > 0 ) {
					String fnm = name.substring( 0,adrs );	// ファイル名(分割後)
					String sub = name.substring( adrs );	// 添え字(アンダーバー含む)

//					String[] values = new String[clmNames.length];
//					System.arraycopy( rowVal,0,values,0,values.length );		// 行にセットするに当たり、rowVal を values にコピーしておく。

					// カラム名で検索しながら、レコード単位になるようにセットしていきます。
					for( int j=0; j<clmNames.length; j++ ) {
						String clm = clmNames[j] ;
						String nm  = null;
						String val = null;

						if( fnm.equalsIgnoreCase( clm ) ) {		// ファイル名カラム(_NEWファイル名も、この値にすでに変わっている）
//							nm  = name;
//							val = multi.getFilesystemName( nm );
							val = fval;
						}
						else if( ( fnm + "_ORG" ).equalsIgnoreCase( clm ) ) {		// 元ファイル名カラム
//							nm   = name + "_ORG" ;
//							val  = multi.getOriginalFileName( name );	// 注意：取得は、送信名
							val  = oval;
						}
						else if( ( fnm + "_NEW" ).equalsIgnoreCase( clm ) ) {		// 新ファイル名カラム
							nm   = name + "_NEW" ;
							val  = multi.getParameter( nm );
						}
						else {
							nm  = clmNames[j] + sub;					// sub は、アンダーバー含む添え字
							// 5.6.5.2 (2013/06/21) チェックボックス配列の値取得を考慮した、MultipartRequest のパラメータ値取得
							val = getParamVal( nm,multi );
						}
						if( val != null ) { values[j] = val ; }		// val が null でないときのみセットします。
//						setRequestAttribute( nm,val );
					}
				}
				table.addColumnValues( values );
			}
		}

		// Attribute で使えるようにセットしておく。
		setRequestAttribute( "directory",fileURL );

		// Attribute で使えるようにセットしておく。
		String[] params = multi.getParameterNames();
		for( int i=0; i<params.length; i++ ) {
			String name = params[i];
			// 5.3.2.0 (2011/02/01) チェック行のパラメーターはint配列側に変換
			if ( HybsSystem.ROW_SEL_KEY.equals( name ) ) {
				setRequestAttribute( name,multi.getIntParameters(name) );
			}
			else {
				// 5.6.5.2 (2013/06/21) チェックボックス配列の値取得を考慮した、MultipartRequest のパラメータ値取得
				String val = getParamVal( name,multi );
				setRequestAttribute( name,val );
			}
		}

		// 5.3.2.0 (2011/02/01) パラメーター名を保存する。
		String[] names = new String[ files.length + params.length ];
	    System.arraycopy( files, 0, names, 0, files.length );
	    System.arraycopy( params, 0, names, files.length, params.length );
	    setParameterNames( names );

		return table ;
	}

	/**
	 * チェックボックス配列の値取得を考慮した、MultipartRequest のパラメータ値取得
	 * 
	 * ここでは、、MultipartRequest のパラメータ値を取得します。
	 * 値の取得で、チェックボックス配列の場合は、取得した値が、"0" の場合のみ
	 * 配列でパラメータを取得し直し、"1" がないかどうか再度検索します。
 	 * チェックボックスでは、チェック時の "1" と、チェックされなかった場合の、
	 * hidden の "0" の両方の値が配列としてリクエストされるケースがある為です。
	 *
	 * @og.rev 5.6.5.2 (2013/06/21) 新規作成
	 *
	 * @param	 key	   MultipartRequest のパラメータ取得のためのキー
	 * @param	 multi	   MultipartRequestオブジェクト
	 *
	 * @return	 チェックボックス配列を考慮したパラメータ値
	 */
	private String getParamVal(  final String key , final MultipartRequest multi ) {
		String val = multi.getParameter( key );

		if( "0".equals(val) ){ // 5.4.4.2 チェックボックス配列対応。
			String[] vals = multi.getParameters( key );
			if( vals != null && vals.length > 1 ) {
				for( int j=0; j<vals.length; j++ ) {
					if( "1".equals( vals[j] ) ) {
						val = "1";
						break;
					}
				}
			}
		}
		return val ;
	}

	/**
	 * 【TAG】ファイルをアップロードするディレクトリを指定します
	 *		(初期値:FILE_URL[={@og.value org.opengion.hayabusa.common.SystemData#FILE_URL}])。
	 *
	 * @og.tag
	 * この属性で指定されるディレクトリに、アップロードされたファイルをセーブします。
	 * 指定方法は、通常の fileURL 属性と同様に、先頭が、'/' (UNIX) または、２文字目が、
	 * ":" (Windows)の場合は、指定のURLそのままのディレクトリに、そうでない場合は、
	 * fileURL = "{&#064;USER.ID}" と指定すると、FILE_URL 属性で指定のフォルダの下に、
	 * さらに、各個人ID別のフォルダを作成して、そこにセーブします。
	 * (初期値:システム定数のFILE_URL[={@og.value org.opengion.hayabusa.common.SystemData#FILE_URL}])。
	 *
	 * @og.rev 4.0.0.0 (2005/01/31) StringUtil.urlAppend メソッドの利用
	 * @og.rev 4.0.0.0 (2007/11/20) 指定されたディレクトリ名の最後が"\"or"/"で終わっていない場合に、"/"を付加する。
	 *
	 * @param	url ファイルURL
	 * @see		org.opengion.hayabusa.common.SystemData#FILE_URL
	 */
	public void setFileURL( final String url ) {
		String furl = nval( getRequestParameter( url ),null );
		if( furl != null ) {
			char ch = furl.charAt( furl.length()-1 );
			if( ch != '/' && ch != '\\' ) { furl = furl + "/"; }
			fileURL = StringUtil.urlAppend( fileURL,furl );
		}
	}

	/**
	 * 【TAG】最大転送サイズ(Byte)を指定します(初期値:10485760)。
	 *
	 * @og.tag
	 * 最大転送サイズを指定します。初期値は、10*1024*1024 = 10MB です。
	 * 指定は、Byte 単位で指定します。
	 * 0,またはマイナスを指定することで、制限チェックを外す（＝無制限）事ができます。
	 *
	 * @og.rev 3.0.1.1 (2003/03/06) maxPostSize の設定バグ修正。
	 * @og.rev 5.6.5.3 (2013/06/28) コメント追加（0,またはマイナスで無制限）
	 *
	 * @param	maxPS 最大転送サイズ
	 */
	public void setMaxPostSize( final String maxPS ) {
		maxPostSize = nval( getRequestParameter( maxPS ),maxPostSize );
	}

	/**
	 * 【TAG】(通常使いません)sessionから所得する DBTableModelオブジェクトの ID。
	 *
	 * @og.tag
	 * 初期値は、HybsSystem.TBL_MDL_KEY です。
	 *
	 * @og.rev 3.5.2.0 (2003/10/20) nvalメソッドを適用するように変更。
	 *
	 * @param	id sessionに登録する時の ID
	 */
	public void setTableId( final String id ) {
		tableId = nval( getRequestParameter( id ),tableId );
	}

	/**
	 * 【TAG】(通常使いません)ファイルを作成するときのファイル名をセットします。
	 *
	 * @og.tag
	 * ファイルを作成するときのファイル名をセットします。
	 * これは、複数同時にアップロードファイル名を変更する時に使用できません。
	 * 通常、アップロードされたファイル名を指定する場合、アップロードするinput タグの
	 * name 属性に指定する名称 ＋ "_NEW" というリクエスト値を同時に送信すれば、
	 * 内部的に関連付けて、ファイル名を更新します。
	 * その場合、クライアントより指定したファイル名は、name属性＋"_ORG" という
	 * リクエスト値として取得することが可能になります。
	 * name属性 には、最終的に設定されたファイル名がセットされています。
	 * いずれの値も、{&#064;name属性＋"_ORG"} や、{&#064;name属性＋"_NEW"}として、
	 * アップロードのオリジナルと変更後のファイル名を取得することが出来ます。
	 *
	 * @og.rev 3.5.4.2 (2003/12/15) ファイル名を指定できるようにします。
	 *
	 * @param   filename ファイル名
	 */
	public void setFilename( final String filename ) {
		this.filename = nval( getRequestParameter( filename ),this.filename );
	}

	/**
	 * 【TAG】DBTableModel作成時に、指定のカラムの"_01"～"_99"の添え字をレコードとして作成します。
	 *
	 * @og.tag
	 * 現状は、"KEY","VALUE","ISFILE" のカラムに、データを縦持ちで作成しています。
	 * これを、横持で作成しますが、カラムの末尾に、"_01"～"_99" までの添え字を
	 * 持つ場合は、これをレコードと認識させます。
	 * アンダーバーがない場合は、カラムだけ作成されます。カラム名と同じリクエストがあれば、
	 * すべてのレコードに同じ値がセットされます。
	 * この処理は、columns 属性を設定した場合のみとします。
	 *
	 * @og.rev 5.6.5.2 (2013/06/21) 新規作成
	 *
	 * @param   clms DBTableModel作成時のカラム列(カンマ区切り文字)
	 */
	public void setColumns( final String clms ) {
		columns = nval( getRequestParameter( clms ),columns );
	}

	/**
	 * 【TAG】ファイルアップロード時に、すでに同名のファイルが存在した場合に、バックアップ処理(renameTo)するかどうか[true/false]を指定します(初期値:false)。
	 *
	 * @og.tag
	 * ファイルアップロード時に、アップロード先に、同名のファイルが存在した場合は、既存機能は、そのまま
	 * 置き換えていましたが、簡易バージョンアップ機能として、useBackup="true" を指定すると、既存のファイルを
	 * リネームして、バックアップファイルを作成します。
	 * バックアップファイルは、アップロードフォルダを基準として、_backup/ファイル名.拡張子_処理時刻のlong値.拡張子 になります。
	 * オリジナルのファイル名（拡張子付）を残したまま、"_処理時刻のlong値" を追加し、さらに、オリジナルの拡張子を追加します。
	 * バックアップファイルの形式は指定できません。
	 * 
	 * 初期値は、互換性を持たせるため、false です。
	 *
	 * @og.rev 5.6.5.3 (2013/06/28) 新規作成
	 *
	 * @param   flag ファイルアップロード時に、バックアップ処理(renameTo)するかどうか[true/false]を指定
	 */
	public void setUseBackup( final String flag ) {
		useBackup = nval( getRequestParameter( flag ),useBackup );
	}

	/**
	 * タグの名称を、返します。
	 * 自分自身のクラス名より、自動的に取り出せないため、このメソッドをオーバーライドします。
	 *
	 * @og.rev 4.0.0.0 (2005/01/31) 新規追加
	 *
	 * @return  タグの名称
	 */
	@Override
	protected String getTagName() {
		return "upload" ;
	}

	/**
	 * このオブジェクトの文字列表現を返します。
	 * 基本的にデバッグ目的に使用します。
	 *
	 * @return このクラスの文字列表現
	 */
	@Override
	public String toString() {
		return org.opengion.fukurou.util.ToString.title( this.getClass().getName() )
				.println( "VERSION"		,VERSION	)
				.println( "names"		,names		)
				.println( "ENCODE"		,ENCODE		)
				.println( "fileURL" 	,fileURL	)
				.println( "filename"    ,filename	)
				.println( "maxPostSize" ,maxPostSize)
				.println( "tableId" 	,tableId	)
				.println( "columns" 	,columns	)				// 5.6.5.2 (2013/06/21)
				.println( "useBackup" 	,useBackup	)				// 5.6.5.3 (2013/06/28)
				.println( "Other..."	,getAttributes().getAttribute() )
				.fixForm().toString() ;
	}
}
